@@ -27,8 +27,10 @@ from boto import handler
 from boto.connection import AWSQueryConnection
 from boto.sdb.domain import Domain, DomainMetaData
 from boto.sdb.item import Item
+from boto.sdb.regioninfo import SDBRegionInfo
 from boto.exception import SDBResponseError
 from boto.resultset import ResultSet
+import warnings
 
 class ItemThread(threading.Thread):
     
@@ -47,18 +49,28 @@ class ItemThread(threading.Thread):
 
 class SDBConnection(AWSQueryConnection):
 
+    DefaultRegionName = 'us-east-1'
+    DefaultRegionEndpoint = 'sdb.amazonaws.com'
     APIVersion = '2007-11-07'
     SignatureVersion = '2'
     ResponseError = SDBResponseError
 
     def __init__(self, aws_access_key_id=None, aws_secret_access_key=None,
                  is_secure=True, port=None, proxy=None, proxy_port=None,
-                 proxy_user=None, proxy_pass=None, host='sdb.amazonaws.com', debug=0,
-                 https_connection_factory=None):
+                 proxy_user=None, proxy_pass=None, debug=0,
+                 https_connection_factory=None, region=None, path='/', converter=None):
+        if not region:
+            region = SDBRegionInfo(self, self.DefaultRegionName, self.DefaultRegionEndpoint)
+        self.region = region
         AWSQueryConnection.__init__(self, aws_access_key_id, aws_secret_access_key,
                                     is_secure, port, proxy, proxy_port, proxy_user, proxy_pass,
-                                    host, debug, https_connection_factory)
+                                    self.region.endpoint, debug, https_connection_factory, path)
         self.box_usage = 0.0
+        self.converter = converter
+        self.item_cls = Item
+
+    def set_item_cls(self, cls):
+        self.item_cls = cls
 
     def build_name_value_list(self, params, attributes, replace=False):
         keys = attributes.keys()
@@ -69,17 +81,50 @@ class SDBConnection(AWSQueryConnection):
             if isinstance(value, list):
                 for v in value:
                     params['Attribute.%d.Name'%i] = key
+                    if self.converter:
+                        v = self.converter.encode(v)
                     params['Attribute.%d.Value'%i] = v
                     if replace:
                         params['Attribute.%d.Replace'%i] = 'true'
                     i += 1
             else:
                 params['Attribute.%d.Name'%i] = key
+                if self.converter:
+                    value = self.converter.encode(value)
                 params['Attribute.%d.Value'%i] = value
                 if replace:
                     params['Attribute.%d.Replace'%i] = 'true'
             i += 1
 
+    def build_batch_list(self, params, items, replace=False):
+        item_names = items.keys()
+        i = 0
+        for item_name in item_names:
+            j = 0
+            item = items[item_name]
+            attr_names = item.keys()
+            params['Item.%d.ItemName' % i] = item_name
+            for attr_name in attr_names:
+                value = item[attr_name]
+                if isinstance(value, list):
+                    for v in value:
+                        if self.converter:
+                            v = self.converter.encode(v)
+                        params['Item.%d.Attribute.%d.Name' % (i,j)] = attr_name
+                        params['Item.%d.Attribute.%d.Value' % (i,j)] = v
+                        if replace:
+                            params['Item.%d.Attribute.%d.Replace' % (i,j)] = 'true'
+                        j += 1
+                else:
+                    params['Item.%d.Attribute.%d.Name' % (i,j)] = attr_name
+                    if self.converter:
+                        value = self.converter.encode(value)
+                    params['Item.%d.Attribute.%d.Value' % (i,j)] = value
+                    if replace:
+                        params['Item.%d.Attribute.%d.Replace' % (i,j)] = 'true'
+                    j += 1
+            i += 1
+
     def build_name_list(self, params, attribute_names):
         i = 1
         attribute_names.sort()
@@ -91,8 +136,8 @@ class SDBConnection(AWSQueryConnection):
         """
         Returns the BoxUsage accumulated on this SDBConnection object.
 
-        @rtype: float
-        @return: The accumulated BoxUsage of all requests made on the connection.
+        :rtype: float
+        :return: The accumulated BoxUsage of all requests made on the connection.
         """
         return self.box_usage
 
@@ -107,18 +152,18 @@ class SDBConnection(AWSQueryConnection):
     def get_domain(self, domain_name, validate=True):
         domain = Domain(self, domain_name)
         if validate:
-            self.query(domain, '', max_items=1)
+            self.select(domain, """select * from `%s` limit 1""" % domain_name)
         return domain
 
     def lookup(self, domain_name, validate=True):
         """
         Lookup an existing SimpleDB domain
 
-        @type domain_name: string
-        @param domain_name: The name of the new domain
+        :type domain_name: string
+        :param domain_name: The name of the new domain
 
-        @rtype: L{Domain<boto.sdb.domain.Domain>} object or None
-        @return: The Domain object or None if the domain does not exist.
+        :rtype: :class:`boto.sdb.domain.Domain` object or None
+        :return: The Domain object or None if the domain does not exist.
         """
         try:
             domain = self.get_domain(domain_name, validate)
@@ -138,11 +183,11 @@ class SDBConnection(AWSQueryConnection):
         """
         Create a SimpleDB domain.
 
-        @type domain_name: string
-        @param domain_name: The name of the new domain
+        :type domain_name: string
+        :param domain_name: The name of the new domain
 
-        @rtype: L{Domain<boto.sdb.domain.Domain>} object
-        @return: The newly created domain
+        :rtype: :class:`boto.sdb.domain.Domain` object
+        :return: The newly created domain
         """
         params = {'DomainName':domain_name}
         d = self.get_object('CreateDomain', params, Domain)
@@ -159,11 +204,11 @@ class SDBConnection(AWSQueryConnection):
         """
         Delete a SimpleDB domain.
 
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @rtype: bool
-        @return: True if successful
+        :rtype: bool
+        :return: True if successful
         
         B{Note:} This will delete the domain and all items within the domain.
         """
@@ -175,11 +220,11 @@ class SDBConnection(AWSQueryConnection):
         """
         Get the Metadata for a SimpleDB domain.
 
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @rtype: L{DomainMetaData<boto.sdb.domain.DomainMetaData>} object
-        @return: The newly created domain metadata object
+        :rtype: :class:`boto.sdb.domain.DomainMetaData` object
+        :return: The newly created domain metadata object
         """
         domain, domain_name = self.get_domain_and_name(domain_or_name)
         params = {'DomainName':domain_name}
@@ -191,22 +236,22 @@ class SDBConnection(AWSQueryConnection):
         """
         Store attributes for a given item in a domain.
 
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @type item_name: string
-        @param item_name: The name of the item whose attributes are being stored.
+        :type item_name: string
+        :param item_name: The name of the item whose attributes are being stored.
 
-        @type attribute_names: dict or dict-like object
-        @param attribute_names: The name/value pairs to store as attributes
+        :type attribute_names: dict or dict-like object
+        :param attribute_names: The name/value pairs to store as attributes
 
-        @type replace: bool
-        @param replace: Whether the attribute values passed in will replace
+        :type replace: bool
+        :param replace: Whether the attribute values passed in will replace
                         existing values or will be added as addition values.
                         Defaults to True.
 
-        @rtype: bool
-        @return: True if successful
+        :rtype: bool
+        :return: True if successful
         """
         domain, domain_name = self.get_domain_and_name(domain_or_name)
         params = {'DomainName' : domain_name,
@@ -214,23 +259,50 @@ class SDBConnection(AWSQueryConnection):
         self.build_name_value_list(params, attributes, replace)
         return self.get_status('PutAttributes', params)
 
+    def batch_put_attributes(self, domain_or_name, items, replace=True):
+        """
+        Store attributes for multiple items in a domain.
+
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
+
+        :type items: dict or dict-like object
+        :param items: A dictionary-like object.  The keys of the dictionary are
+                      the item names and the values are themselves dictionaries
+                      of attribute names/values, exactly the same as the
+                      attribute_names parameter of the scalar put_attributes
+                      call.
+
+        :type replace: bool
+        :param replace: Whether the attribute values passed in will replace
+                        existing values or will be added as addition values.
+                        Defaults to True.
+
+        :rtype: bool
+        :return: True if successful
+        """
+        domain, domain_name = self.get_domain_and_name(domain_or_name)
+        params = {'DomainName' : domain_name}
+        self.build_batch_list(params, items, replace)
+        return self.get_status('BatchPutAttributes', params, verb='POST')
+
     def get_attributes(self, domain_or_name, item_name, attribute_names=None, item=None):
         """
         Retrieve attributes for a given item in a domain.
 
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @type item_name: string
-        @param item_name: The name of the item whose attributes are being retrieved.
+        :type item_name: string
+        :param item_name: The name of the item whose attributes are being retrieved.
 
-        @type attribute_names: string or list of strings
-        @param attribute_names: An attribute name or list of attribute names.  This
+        :type attribute_names: string or list of strings
+        :param attribute_names: An attribute name or list of attribute names.  This
                                 parameter is optional.  If not supplied, all attributes
                                 will be retrieved for the item.
 
-        @rtype: L{Item<boto.sdb.item.Item>}
-        @return: An Item mapping type containing the requested attribute name/values
+        :rtype: :class:`boto.sdb.item.Item`
+        :return: An Item mapping type containing the requested attribute name/values
         """
         domain, domain_name = self.get_domain_and_name(domain_or_name)
         params = {'DomainName' : domain_name,
@@ -243,7 +315,7 @@ class SDBConnection(AWSQueryConnection):
         body = response.read()
         if response.status == 200:
             if item == None:
-                item = Item(domain, item_name)
+                item = self.item_cls(domain, item_name)
             h = handler.XmlHandler(item, self)
             xml.sax.parseString(body, h)
             return item
@@ -254,21 +326,21 @@ class SDBConnection(AWSQueryConnection):
         """
         Delete attributes from a given item in a domain.
 
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @type item_name: string
-        @param item_name: The name of the item whose attributes are being deleted.
+        :type item_name: string
+        :param item_name: The name of the item whose attributes are being deleted.
 
-        @type attributes: dict, list or L{Item<boto.sdb.item.Item>}
-        @param attributes: Either a list containing attribute names which will cause
+        :type attributes: dict, list or :class:`boto.sdb.item.Item`
+        :param attributes: Either a list containing attribute names which will cause
                            all values associated with that attribute name to be deleted or
                            a dict or Item containing the attribute names and keys and list
                            of values to delete as the value.  If no value is supplied,
                            all attribute name/values for the item will be deleted.
                            
-        @rtype: bool
-        @return: True if successful
+        :rtype: bool
+        :return: True if successful
         """
         domain, domain_name = self.get_domain_and_name(domain_or_name)
         params = {'DomainName':domain_name,
@@ -276,7 +348,7 @@ class SDBConnection(AWSQueryConnection):
         if attr_names:
             if isinstance(attr_names, list):
                 self.build_name_list(params, attr_names)
-            elif isinstance(attr_names, dict) or isinstance(attr_names, Item):
+            elif isinstance(attr_names, dict) or isinstance(attr_names, self.item_cls):
                 self.build_name_value_list(params, attr_names)
         return self.get_status('DeleteAttributes', params)
         
@@ -284,20 +356,21 @@ class SDBConnection(AWSQueryConnection):
         """
         Returns a list of item names within domain_name that match the query.
         
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @type query: string
-        @param query: The SimpleDB query to be performed.
+        :type query: string
+        :param query: The SimpleDB query to be performed.
 
-        @type max_items: int
-        @param max_items: The maximum number of items to return.  If not
+        :type max_items: int
+        :param max_items: The maximum number of items to return.  If not
                           supplied, the default is None which returns all
                           items matching the query.
 
-        @rtype: ResultSet
-        @return: An iterator containing the results.
+        :rtype: ResultSet
+        :return: An iterator containing the results.
         """
+        warnings.warn('Query interface is deprecated', DeprecationWarning)
         domain, domain_name = self.get_domain_and_name(domain_or_name)
         params = {'DomainName':domain_name,
                   'QueryExpression' : query}
@@ -312,25 +385,26 @@ class SDBConnection(AWSQueryConnection):
         """
         Returns a set of Attributes for item names within domain_name that match the query.
         
-        @type domain_or_name: string or L{Domain<boto.sdb.domain.Domain>} object.
-        @param domain_or_name: Either the name of a domain or a Domain object
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
 
-        @type query: string
-        @param query: The SimpleDB query to be performed.
+        :type query: string
+        :param query: The SimpleDB query to be performed.
 
-        @type attr_names: list
-        @param attr_names: The name of the attributes to be returned.
+        :type attr_names: list
+        :param attr_names: The name of the attributes to be returned.
                            If no attributes are specified, all attributes
                            will be returned.
 
-        @type max_items: int
-        @param max_items: The maximum number of items to return.  If not
+        :type max_items: int
+        :param max_items: The maximum number of items to return.  If not
                           supplied, the default is None which returns all
                           items matching the query.
 
-        @rtype: ResultSet
-        @return: An iterator containing the results.
+        :rtype: ResultSet
+        :return: An iterator containing the results.
         """
+        warnings.warn('Query interface is deprecated', DeprecationWarning)
         domain, domain_name = self.get_domain_and_name(domain_or_name)
         params = {'DomainName':domain_name,
                   'QueryExpression' : query}
@@ -340,7 +414,31 @@ class SDBConnection(AWSQueryConnection):
             params['NextToken'] = next_token
         if attr_names:
             self.build_list_params(params, attr_names, 'AttributeName')
-        return self.get_list('QueryWithAttributes', params, [('Item', Item)], parent=domain)
+        return self.get_list('QueryWithAttributes', params, [('Item', self.item_cls)], parent=domain)
+
+    def select(self, domain_or_name, query='', next_token=None):
+        """
+        Returns a set of Attributes for item names within domain_name that match the query.
+        The query must be expressed in using the SELECT style syntax rather than the
+        original SimpleDB query language.
+        Even though the select request does not require a domain object, a domain
+        object must be passed into this method so the Item objects returned can
+        point to the appropriate domain.
+        
+        :type domain_or_name: string or :class:`boto.sdb.domain.Domain` object.
+        :param domain_or_name: Either the name of a domain or a Domain object
+
+        :type query: string
+        :param query: The SimpleDB query to be performed.
+
+        :rtype: ResultSet
+        :return: An iterator containing the results.
+        """
+        domain, domain_name = self.get_domain_and_name(domain_or_name)
+        params = {'SelectExpression' : query}
+        if next_token:
+            params['NextToken'] = next_token
+        return self.get_list('Select', params, [('Item', self.item_cls)], parent=domain)
 
     def threaded_query(self, domain_or_name, query='', max_items=None, next_token=None, num_threads=6):
         """
